@@ -1,4 +1,4 @@
-# Copyright (c) 2003-2007 LOGILAB S.A. (Paris, FRANCE).
+# Copyright (c) 2003-2009 LOGILAB S.A. (Paris, FRANCE).
 # http://www.logilab.fr/ -- mailto:contact@logilab.fr
 #
 # This program is free software; you can redistribute it and/or modify it under
@@ -19,10 +19,11 @@ from __future__ import generators
 
 from logilab.common.compat import set
 from logilab import astng
+from logilab.astng.infutils import YES, Instance, are_exclusive
 
 from pylint.interfaces import IASTNGChecker
 from pylint.checkers import BaseChecker
-from pylint.checkers.utils import overrides_a_method
+from pylint.checkers.utils import PYMETHODS, overrides_a_method
 
 MSGS = {
     'F0202': ('Unable to check methods signature (%s / %s)',
@@ -175,6 +176,10 @@ instance attributes.'}
         # checks attributes are defined in an allowed method such as __init__
         defining_methods = self.config.defining_attr_methods
         for attr, nodes in cnode.instance_attrs.items():
+            nodes = [n for n in nodes if not
+                    isinstance(n.statement(), (astng.Delete, astng.AugAssign))]
+            if not nodes:
+                continue # error detected by typechecking
             node = nodes[0] # XXX
             frame = node.frame()
             if frame.name not in defining_methods:
@@ -232,13 +237,6 @@ instance attributes.'}
         except astng.NotFoundError:
             pass
         
-    pymethods = set(('__new__', '__init__',
-                     '__getattr__', '__setattr__',
-                     '__hash__', '__cmp__',
-                     '__mul__', '__div__', '__add__', '__sub__',
-                     '__rmul__', '__rdiv__', '__radd__', '__rsub__',
-                     # To be continued
-                     ))
     def leave_function(self, node):
         """on method node, check if this method couldn't be a function
         
@@ -247,11 +245,11 @@ instance attributes.'}
         kind of method defined in an interface for this warning
         """
         if node.is_method():
-            if node.argnames is not None:
+            if node.args.args is not None:
                 self._first_attrs.pop()
             class_node = node.parent.frame()
             if (self._meth_could_be_func and node.type == 'method'
-                and not node.name in self.pymethods
+                and not node.name in PYMETHODS
                 and not (node.is_abstract() or
                          overrides_a_method(class_node, node.name))
                 and class_node.type != 'interface'):
@@ -283,8 +281,8 @@ instance attributes.'}
             if klass is None or not (callee == klass.name or
                 callee in klass.basenames
                 or (isinstance(node.expr, astng.CallFunc)
-                    and isinstance(node.expr.node, astng.Name) 
-                    and node.expr.node.name == 'super')):
+                    and isinstance(node.expr.func, astng.Name) 
+                    and node.expr.func.name == 'super')):
                 self.add_message('W0212', node=node, args=attrname)
             
             
@@ -302,10 +300,6 @@ instance attributes.'}
         for attr, nodes in accessed.items():
             # deactivate "except doesn't do anything", that's expected
             # pylint: disable-msg=W0704
-##             # is it a builtin attribute ?
-##             if attr in ('__dict__', '__class__', '__doc__'):
-##                 # FIXME: old class object doesn't have __class__
-##                 continue
             # is it a class attribute ?
             try:
                 node.local_attr(attr)
@@ -322,20 +316,19 @@ instance attributes.'}
                 pass
             # is it an instance attribute ?
             try:
-                def_nodes = node.instance_attr(attr) # XXX
-                #instance_attribute = True
+                defstmts = node.instance_attr(attr) 
             except astng.NotFoundError:
                 pass
-                #instance_attribute = False
             else:
-                if len(def_nodes) == 1:
-                    def_node = def_nodes[0]
+                if len(defstmts) == 1:
+                    defstmt = defstmts[0]
                     # check that if the node is accessed in the same method as
                     # it's defined, it's accessed after the initial assigment
-                    frame = def_node.frame()
-                    lno = def_node.source_line()
+                    frame = defstmt.frame()
+                    lno = defstmt.fromlineno
                     for _node in nodes:
-                        if _node.frame() is frame and _node.lineno < lno:
+                        if _node.frame() is frame and _node.fromlineno < lno \
+                           and not are_exclusive(_node.statement(), defstmt, ('AttributeError', 'Exception', 'BaseException')):
                             self.add_message('E0203', node=_node,
                                              args=(attr, lno))
         
@@ -348,27 +341,29 @@ instance attributes.'}
         * not one of the above for a static method
         """
         # don't care about functions with unknown argument (builtins)
-        if node.argnames is None:
+        if node.args.args is None:
             return
-        self._first_attrs.append(node.argnames and node.argnames[0])
+        first_arg = node.args.args and node.argnames()[0]
+        self._first_attrs.append(first_arg)
+        first = self._first_attrs[-1]
         # static method
         if node.type == 'staticmethod':
-            if node.argnames and node.argnames[0] in ('self', 'cls', 'mcs'):
-                self.add_message('W0211', args=node.argnames[0], node=node)
+            if first_arg in ('self', 'cls', 'mcs'):
+                self.add_message('W0211', args=first, node=node)
             self._first_attrs[-1] = None
         # class / regular method with no args
-        elif not node.argnames:
+        elif not node.args.args:
             self.add_message('E0211', node=node)
         # metaclass method
         elif metaclass:
-            if self._first_attrs[-1] != 'mcs':
+            if first != 'mcs':
                 self.add_message('C0203', node=node)
         # class method
         elif node.type == 'classmethod':
-            if self._first_attrs[-1] != 'cls':
+            if first != 'cls':
                 self.add_message('C0202', node=node)
         # regular method without self as argument
-        elif self._first_attrs[-1] != 'self':
+        elif first != 'self':
             self.add_message('E0213', node=node)
 
     def _check_bases_classes(self, node):
@@ -423,7 +418,7 @@ instance attributes.'}
         except astng.InferenceError:
             if e0221_hack[0]:
                 return
-            implements = astng.Instance(node).getattr('__implements__')[0]
+            implements = Instance(node).getattr('__implements__')[0]
             assignment = implements.parent
             assert isinstance(assignment, astng.Assign)
             # assignment.expr can be a Name or a Tuple or whatever.
@@ -431,7 +426,7 @@ instance attributes.'}
             # FIXME: in case of multiple interfaces, find which one could not
             #        be resolved
             self.add_message('F0220', node=implements,
-                             args=(node.name, assignment.expr.as_string()))
+                             args=(node.name, assignment.value.as_string()))
 
     def _check_init(self, node):
         """check that the __init__ method call super or ancestors'__init__
@@ -440,18 +435,18 @@ instance attributes.'}
         klass_node = node.parent.frame()        
         to_call = _ancestors_to_call(klass_node)
         for stmt in node.nodes_of_class(astng.CallFunc):
-            expr = stmt.node
+            expr = stmt.func
             if not isinstance(expr, astng.Getattr) \
                    or expr.attrname != '__init__':
                 continue
             # skip the test if using super
             if isinstance(expr.expr, astng.CallFunc) and \
-               isinstance(expr.expr.node, astng.Name) and \
-               expr.expr.node.name == 'super':
+               isinstance(expr.expr.func, astng.Name) and \
+               expr.expr.func.name == 'super':
                 return
             try:
                 klass = expr.expr.infer().next()
-                if klass is astng.YES:
+                if klass is YES:
                     continue
                 try:
                     del to_call[klass]
@@ -474,11 +469,11 @@ instance attributes.'}
             self.add_message('F0202', args=(method1, refmethod), node=method1)
             return
         # don't care about functions with unknown argument (builtins)
-        if method1.argnames is None or refmethod.argnames is None:
+        if method1.args.args is None or refmethod.args.args is None:
             return
-        if len(method1.argnames) != len(refmethod.argnames):
+        if len(method1.args.args) != len(refmethod.args.args):
             self.add_message('W0221', args=class_type, node=method1)
-        elif len(method1.defaults) < len(refmethod.defaults):
+        elif len(method1.args.defaults) < len(refmethod.args.defaults):
             self.add_message('W0222', args=class_type, node=method1)
 
                         
@@ -500,10 +495,10 @@ def node_method(node, method_name):
     """get astng for <method_name> on the given class node, ensuring it
     is a Function node
     """
-    stmt = node.local_attr(method_name)
-    if not isinstance(stmt, astng.Function):
-        raise astng.NotFoundError(method_name)
-    return stmt
+    for n in node.local_attr(method_name):
+        if isinstance(n, astng.Function):
+            return n
+    raise astng.NotFoundError(method_name)
         
 def register(linter):
     """required method to auto register this checker """
