@@ -20,9 +20,7 @@
  FIXME: missing 13, 15, 16
 """
 
-__revision__ = "$Id: design_analysis.py,v 1.11 2005-12-30 15:41:29 adim Exp $"
-
-from logilab.astng import Function, InferenceError
+from logilab.astng import Function, If, InferenceError
 
 from pylint.interfaces import IASTNGChecker
 from pylint.checkers import BaseChecker
@@ -45,8 +43,8 @@ MSGS = {
     'R0902': ('Too many instance attributes (%s/%s)',
               'Used when class has too many instance attributes, try to reduce \
               this to get a more simple (and so easier to use) class.'),
-    'R0903': ('To few public methods (%s/%s)',
-              'Used when class has to few public methods, so be sure it\'s \
+    'R0903': ('Too few public methods (%s/%s)',
+              'Used when class has too few public methods, so be sure it\'s \
               really worth it.'),
     'R0904': ('Too many public methods (%s/%s)',
               'Used when class has too many public methods, try to reduce \
@@ -241,7 +239,7 @@ class MisdesignChecker(BaseChecker):
         self._returns.append(0)
         self._branchs.append(0)
         # check number of arguments
-        args = node.argnames
+        args = node.args.args
         if args is not None and len(args) > self.config.max_args:
             self.add_message('R0913', node=node,
                              args=(len(args), self.config.max_args))
@@ -272,23 +270,27 @@ class MisdesignChecker(BaseChecker):
 
     def visit_return(self, _):
         """count number of returns/yields"""
+        if not self._returns:
+            return # return outside function, reported by the base checker
         self._returns[-1] += 1
         
     def visit_yield(self, _):
         """count number of returns/yields"""
+        if not self._returns:
+            return # yield outside function, reported by the base checker
         self._returns[-1] += 1
         
     def visit_default(self, node):
         """default visit method -> increments the statements counter if
         necessary
         """
-        if node.is_statement():
+        if node.is_statement:
             self._stmts += 1
 
     def visit_tryexcept(self, node):
         """increments the branchs counter"""
         branchs = len(node.handlers)
-        if node.else_:
+        if node.orelse:
             branchs += 1
         self._inc_branch(branchs)
         self._stmts += branchs
@@ -300,8 +302,10 @@ class MisdesignChecker(BaseChecker):
         
     def visit_if(self, node):
         """increments the branchs counter"""
-        branchs = len(node.tests)
-        if node.else_:
+        branchs = 1
+        # don't double count If nodes coming from some 'elif'
+        if node.orelse and (len(node.orelse)>1 or
+                            not isinstance(node.orelse[0], If)):
             branchs += 1
         self._inc_branch(branchs)
         self._stmts += branchs
@@ -309,7 +313,7 @@ class MisdesignChecker(BaseChecker):
     def visit_while(self, node):
         """increments the branchs counter"""
         branchs = 1
-        if node.else_:
+        if node.orelse:
             branchs += 1
         self._inc_branch(branchs)
         
@@ -322,7 +326,7 @@ class MisdesignChecker(BaseChecker):
             branchs[i] += branchsnum
 
     # FIXME: make a nice report...
-        
+
 def register(linter):
     """required method to auto register this checker """
     linter.register_checker(MisdesignChecker(linter))
