@@ -1,6 +1,6 @@
 # pylint: disable-msg=W0611
 #
-# Copyright (c) 2003-2007 LOGILAB S.A. (Paris, FRANCE).
+# Copyright (c) 2003-2009 LOGILAB S.A. (Paris, FRANCE).
 # http://www.logilab.fr/ -- mailto:contact@logilab.fr
 #
 # This program is free software; you can redistribute it and/or modify it under
@@ -18,17 +18,15 @@
 """some functions that may be usefull for various checkers
 """
 
-from logilab.common import flatten
-
 from logilab import astng
-from logilab.astng.utils import are_exclusive
+
 try:
     # python >= 2.4
     COMP_NODE_TYPES = (astng.ListComp, astng.GenExpr)
-    FOR_NODE_TYPES = (astng.For, astng.ListCompFor, astng.GenExprFor)
+    FOR_NODE_TYPES = (astng.For, astng.Comprehension, astng.Comprehension)
 except AttributeError:
     COMP_NODE_TYPES = astng.ListComp
-    FOR_NODE_TYPES = (astng.For, astng.ListCompFor)
+    FOR_NODE_TYPES = (astng.For, astng.Comprehension)
 
 def safe_infer(node):
     """return the infered value for the given node.
@@ -56,23 +54,21 @@ def is_super(node):
 
 def is_error(node):
     """return true if the function does nothing but raising an exception"""
-    for child_node in node.code.getChildNodes():
+    for child_node in node.get_children():
         if isinstance(child_node, astng.Raise):
             return True
         return False
 
-def is_raising(stmt):
-    """return true if the given statement node raise an exception
-    """
-    for node in stmt.nodes:
+def is_raising(body):
+    """return true if the given statement node raise an exception"""
+    for node in body:
         if isinstance(node, astng.Raise):
             return True
     return False
 
-def is_empty(node):
+def is_empty(body):
     """return true if the given node does nothing but 'pass'"""
-    children = node.getChildNodes()
-    return len(children) == 1 and isinstance(children[0], astng.Pass)
+    return len(body) == 1 and isinstance(body[0], astng.Pass)
 
 builtins = __builtins__.copy()
 SPECIAL_BUILTINS = ('__builtins__',) # '__path__', '__file__')
@@ -99,14 +95,14 @@ def is_defined_before(var_node, comp_node_types=COMP_NODE_TYPES):
                 if ass_node.name == varname:
                     return True
         elif isinstance(_node, astng.For):
-            for ass_node in _node.assign.nodes_of_class(astng.AssName):
+            for ass_node in _node.target.nodes_of_class(astng.AssName):
                 if ass_node.name == varname:
                     return True
         elif isinstance(_node, astng.With):
             if _node.vars.name == varname:
                 return True
         elif isinstance(_node, (astng.Lambda, astng.Function)):
-            if varname in flatten(_node.argnames):
+            if _node.args.is_argument(varname):
                 return True
             if getattr(_node, 'name', None) == varname:
                 return True
@@ -115,8 +111,8 @@ def is_defined_before(var_node, comp_node_types=COMP_NODE_TYPES):
     # possibly multiple statements on the same line using semi colon separator
     stmt = var_node.statement()
     _node = stmt.previous_sibling()
-    lineno = stmt.lineno
-    while _node and _node.lineno == lineno:
+    lineno = stmt.fromlineno
+    while _node and _node.fromlineno == lineno:
         for ass_node in _node.nodes_of_class(astng.AssName):
             if ass_node.name == varname:
                 return True
@@ -127,25 +123,27 @@ def is_defined_before(var_node, comp_node_types=COMP_NODE_TYPES):
     return False
 
 def is_func_default(node):
-    """return true if the name is used in function default argument's value
+    """return true if the given Name node is used in function default argument's
+    value
     """
-    parent = node.parent
-    if parent is None:
-        return 0
-    if isinstance(parent, astng.Function) and parent.defaults and \
-           node in parent.defaults:
-        return 1
-    return is_func_default(parent)
+    parent = node.scope()
+    if isinstance(parent, astng.Function):
+        for default_node in parent.args.defaults:
+            for default_name_node in default_node.nodes_of_class(astng.Name):
+                if default_name_node is node:
+                    return True
+    return False
 
 def is_func_decorator(node):
-    """return true if the name is used in function decorator
-    """
+    """return true if the name is used in function decorator"""
     parent = node.parent
-    if parent is None:
-        return 0
-    if isinstance(parent, astng.Decorators):
-        return 1
-    return is_func_decorator(parent)
+    while parent is not None:
+        if isinstance(parent, astng.Decorators):
+            return True
+        if parent.is_statement:
+            break
+        parent = parent.parent
+    return False
 
 def is_ancestor_name(frame, node):
     """return True if `frame` is a astng.Class node with `node` in the
@@ -161,12 +159,11 @@ def is_ancestor_name(frame, node):
     return False
 
 def assign_parent(node):
-    """return the higher parent which is not an AssName, AssTuple or AssList
-    node
+    """return the higher parent which is not an AssName, Tuple or List node
     """
     while node and isinstance(node, (astng.AssName,
-                                     astng.AssTuple,
-                                     astng.AssList)):
+                                     astng.Tuple,
+                                     astng.List)):
         node = node.parent
     return node
 
@@ -192,3 +189,18 @@ def display_type(node):
     elif isinstance(node, astng.Module):
         return 'Module'
     return 'Class'
+
+PYMETHODS = set(('__new__', '__init__',
+                 '__len__', '__iter__',
+                 '__getitem__', '__setitem__', '__delitem__',
+                 '__getattribute__', '__getattr__', '__setattr__', '__delattr__',
+                 '__cmp__', '__ge__', '__gt__', '__le__', '__lt__', '__eq__',
+                 '__nonzero__',
+                 '__mul__', '__imul__', '__rmul__',
+                 '__div__', '__idiv__', '__rdiv__',
+                 '__add__', '__iadd__', '__radd__',
+                 '__sub__', '__isub__', '__rsub__',
+                 '__pow__', '__ipow__', '__rpow__',
+                 '__mod__', '__imod__', '__rmod__',
+                 # XXX To be continued
+                 ))
