@@ -21,18 +21,19 @@
 import pickle
 import os
 import sys
-from os.path import exists, isfile, join, expanduser, abspath
+from os.path import exists, isfile, join, expanduser, abspath, dirname
 
 # pylint home is used to save old runs results ################################
 
+USER_HOME = expanduser('~')
 if os.environ.has_key('PYLINTHOME'):
     PYLINT_HOME = os.environ['PYLINTHOME']
-else:
-    USER_HOME = expanduser('~')
     if USER_HOME == '~':
-        PYLINT_HOME = ".pylint.d"
-    else:
-        PYLINT_HOME = join(USER_HOME, '.pylint.d')
+        USER_HOME = dirname(PYLINT_HOME)
+elif USER_HOME == '~':
+    PYLINT_HOME = ".pylint.d"
+else:
+    PYLINT_HOME = join(USER_HOME, '.pylint.d')
         
 if not exists(PYLINT_HOME):
     try:
@@ -103,7 +104,7 @@ The following environment variables are used :
     * PYLINTHOME                                                               
     path to the directory where data of persistent run will be stored. If not
 found, it defaults to ~/.pylint.d/ or .pylint.d (in the current working
-directory) . The current PYLINTHOME is %(PYLINT_HOME)s.                    
+directory) . The current PYLINTHOME is %(PYLINT_HOME)s.        
     * PYLINTRC                                                                 
     path to the configuration file. If not found, it will use the first        
 existant file in ~/.pylintrc, /etc/pylintrc. The current PYLINTRC is
