@@ -3,7 +3,7 @@
 if [ $@ ] ; then
     PYVERSIONS=$@
 else
-    PYVERSIONS="2.2" "2.3" "2.4"
+    PYVERSIONS=`pyversions -iv`
 fi
 
 for ver in $PYVERSIONS; do  
@@ -12,6 +12,6 @@ for ver in $PYVERSIONS; do
     echo "^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^"
     python$ver runtests.py
     echo "^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^"
-    echo -OO
+    echo `python$ver -V` -OO
     python$ver -OO runtests.py
-done
\ No newline at end of file
+done
