@@ -12,4 +12,4 @@ class Aaaa:
     
     def meth2(self):
         """hehehe"""
-        return 'Yo'
+        return 'Yo', self
